@@ -6,8 +6,8 @@
 #' @export
 #'
 #' @examples
-#' fisher_r2z(0.5)
-fisher_z2r <- function(z){
+#' fisher_z_to_r(0.5)
+fisher_z_to_r <- function(z){
   r <- z2r(z)
   return(r)
 }
